@@ -14,19 +14,18 @@
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef PLAYLISTITEM_H
-#define PLAYLISTITEM_H
-
-#include <k3listview.h>
-#include <ksharedptr.h>
-#include <kdebug.h>
+#ifndef JUK_PLAYLISTITEM_H
+#define JUK_PLAYLISTITEM_H
 
+#include <QExplicitlySharedDataPointer>
 #include <QVector>
 #include <QPixmap>
 #include <QList>
+#include <QTreeWidgetItem>
 
 #include "tagguesser.h"
 #include "filehandle.h"
+#include "juk_debug.h"
 
 class Playlist;
 class PlaylistItem;
@@ -43,7 +42,7 @@ typedef QList<PlaylistItem *> PlaylistItemList;
  * Playlist::clearItem().
  */
 
-class PlaylistItem : public K3ListViewItem
+class PlaylistItem : public QTreeWidgetItem
 {
     friend class Playlist;
     friend class SearchPlaylist;
@@ -103,7 +102,6 @@ public:
 
     void setPlaying(bool playing = true, bool master = true);
 
-    virtual void setSelected(bool selected);
     void guessTagInfo(TagGuesser::Type type);
 
     Playlist *playlist() const;
@@ -145,12 +143,12 @@ public:
     /**
      * Returns properly casted item below this one.
      */
-    PlaylistItem *itemBelow() { return static_cast<PlaylistItem *>(K3ListViewItem::itemBelow()); }
+    PlaylistItem *itemBelow() { return static_cast<PlaylistItem *>(treeWidget()->itemBelow(this)); }
 
     /**
      * Returns properly casted item above this one.
      */
-    PlaylistItem *itemAbove() { return static_cast<PlaylistItem *>(K3ListViewItem::itemAbove()); }
+    PlaylistItem *itemAbove() { return static_cast<PlaylistItem *>(treeWidget()->itemAbove(this)); }
 
     /**
      * Returns a reference to the list of the currnetly playing items, with the
@@ -165,7 +163,7 @@ protected:
      * subclass or friend class.
      */
     PlaylistItem(CollectionListItem *item, Playlist *parent);
-    PlaylistItem(CollectionListItem *item, Playlist *parent, Q3ListViewItem *after);
+    PlaylistItem(CollectionListItem *item, Playlist *parent, QTreeWidgetItem *after);
 
     /**
      * This is the constructor that shold be used by subclasses.
@@ -178,10 +176,10 @@ protected:
      */
     virtual ~PlaylistItem();
 
-    virtual void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);
-    virtual void paintFocus(QPainter *, const QColorGroup &, const QRect &) {}
+    //virtual void paintCell(QPainter *p, const QColorGroup &cg, int column, int width, int align);
+    //virtual void paintFocus(QPainter *, const QColorGroup &, const QRect &) {}
 
-    virtual int compare(Q3ListViewItem *item, int column, bool ascending) const;
+    virtual int compare(QTreeWidgetItem *item, int column, bool ascending) const;
     int compare(const PlaylistItem *firstItem, const PlaylistItem *secondItem, int column, bool ascending) const;
 
     bool isValid() const;
@@ -192,24 +190,20 @@ protected:
      * Shared data between all PlaylistItems from the same track (incl. the CollectionItem
      * representing said track.
      */
-    struct Data : public KShared
+    struct Data : public QSharedData
     {
-        Data() {}
-        Data(const QFileInfo &info, const QString &path) : fileHandle(info, path) {}
-        Data(const QString &path) : fileHandle(path) {}
-
-        FileHandle fileHandle;
+        FileHandle fileHandle; // Set within CollectionList
         QVector<QString> metadata; ///< Artist, album, or genre tags.  Other columns unfilled
         QVector<int> cachedWidths;
     };
 
-    KSharedPtr<Data> data() const { return d; }
+    using DataPtr = QExplicitlySharedDataPointer<Data>;
+    DataPtr sharedData() const { return d; }
 
 private:
-    KSharedPtr<Data> d;
+    DataPtr d;
 
     void setup(CollectionListItem *item);
-    using Q3ListViewItem::setup; // Avoid warning about hidden function.
 
     CollectionListItem *m_collectionItem;
     quint32 m_trackId;
@@ -219,11 +213,7 @@ private:
 
 inline QDebug operator<<(QDebug s, const PlaylistItem &item)
 {
-    if(&item == 0)
-        s << "(nil)";
-    else
-        s << item.text(PlaylistItem::TrackColumn);
-
+    s << item.text(PlaylistItem::TrackColumn);
     return s;
 }
 
